OC.L10N.register(
    "sms_relentless",
    {
    "Settings" : "Settings",
    "SMS Relentless" : "SMS Relentless",
    "Enter your credentials and options in the fields from below. As SMS service provider you can use Telnyx, Plivo, Twilio, Flowroute or all of them:" : "Enter your credentials and options in the fields from below. As SMS service provider you can use Telnyx, Plivo, Twilio, Flowroute or all of them:",
    "Generate new webhook URL for incoming SMS" : "Generate new webhook URL for incoming SMS",
    "Generate new webhook URL for delivery receipts" : "Generate new webhook URL for delivery receipts",
    "Generate new webhook URL for incoming SMS" : "Generate new webhook URL for incoming SMS",
    "Generate new webhook URL for delivery receipts" : "Generate new webhook URL for delivery receipts",
    "Save" : "Save",
    "New SMS message" : "New SMS message",
    "New SMS message from SMS Relentless" : "New SMS message from SMS Relentless"
},
"nplurals=2; plural=(n != 1);");
