<?php

// autoload_psr4.php @generated by Composer

$vendorDir = dirname(__DIR__);
$baseDir = dirname($vendorDir);

return array(
    'Telnyx\\' => array($baseDir . '/lib', $vendorDir . '/telnyx/telnyx-php/lib'),
);
